/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package pe.siso.academia.MySqlDAO;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import org.json.JSONObject;
import org.json.JSONArray;
import pe.siso.academia.DAO.PermisosDAO;
import pe.siso.academia.utilities.GeneralVariables;
import pe.siso.academia.utilities.ResponseHelper;

/**
 *
 * @author Felipe Escala
 */
public class MySqlPermisosDAO implements PermisosDAO {

	@Override
	public JSONObject listarPermisos(JSONObject datos) throws Exception {
		JSONObject jreturn;
		JSONArray lista = new JSONArray();
		JSONObject data = new JSONObject();
		Connection con = null;
		PreparedStatement pst = null;
		ResultSet rSet = null;
		int rs = 0;
		String sql = "";
		String condicion = "";
		ResponseHelper response = new ResponseHelper();
		try {
			con = MySqlDAOFactory.obtenerConexion(GeneralVariables.nameDB);
			if (datos.has("fechaBusqueda")) {
				condicion = "AND STR_TO_DATE(?,'%d/%m/%Y') BETWEEN p.fecha_inicio_permiso  "
					+ "AND p.fecha_fin_permiso ";
			} else if (datos.has("estudianteBusqueda")) {
				condicion = " AND CONCAT_WS(' ', e.paterno, e.materno, e.nombres) LIKE ? ";
			}
			sql = " SELECT  "
				+ "	p.codigo_permiso,  "
				+ "	CONCAT_WS(' ', e.paterno, e.materno, e.nombres),  "
				+ "	p.descripcion_permiso,  "
				+ "	DATE_FORMAT(p.fecha_inicio_permiso, '%d/%m/%Y'),  "
				+ "	DATE_FORMAT(p.fecha_fin_permiso, '%d/%m/%Y'), "
				+ "    IF(DATE(NOW()) BETWEEN p.fecha_inicio_permiso AND p.fecha_fin_permiso, 1, 0) "
				+ " FROM permisos AS p "
				+ " INNER JOIN estudiante AS e  "
				+ " ON e.codestudiante = p.codigo_estudiante "
				+ " WHERE p.estado_registro = 1 "
				+ " AND e.codsede = ? " + condicion
				+ " ORDER BY p.codigo_permiso DESC "
				+ " LIMIT ?,? ";
			pst = con.prepareStatement(sql);
			int c = 1;
			pst.setInt(c++, datos.getInt("codigoSede"));
			if (datos.has("fechaBusqueda")) {
				pst.setString(c++, datos.getString("fechaBusqueda"));
			} else if (datos.has("estudianteBusqueda")) {
				pst.setString(c++, "%" + datos.getString("estudianteBusqueda") + "%");
			}
			pst.setInt(c++, datos.getInt("start"));
			pst.setInt(c++, datos.getInt("length"));
			rSet = pst.executeQuery();
			int conta = datos.getInt("start") + 1;
			while (rSet.next()) {
				JSONObject obj = new JSONObject();
				c = 1;
				obj.put("numeral", conta++);
				obj.put("codigoPermiso", rSet.getInt(c++));
				obj.put("nombreEstudiante", rSet.getString(c++));
				obj.put("descripcionPermiso", rSet.getString(c++));
				obj.put("inicioPermiso", rSet.getString(c++));
				obj.put("finPermiso", rSet.getString(c++));
				obj.put("vigenciaPermiso", rSet.getInt(c++));
				lista.put(obj);
			}
			if (lista.length() >= 1) {
				sql = " SELECT  "
					+ " COUNT(*) "
					+ " FROM permisos AS p "
					+ " INNER JOIN estudiante AS e  "
					+ " ON e.codestudiante = p.codigo_estudiante "
					+ " WHERE p.estado_registro = 1 "
					+ " AND e.codsede = ? " + condicion
					+ " ORDER BY p.codigo_permiso DESC ";
				pst = con.prepareStatement(sql);
				c = 1;
				pst.setInt(c++, datos.getInt("codigoSede"));
				if (datos.has("fechaBusqueda")) {
					pst.setString(c++, datos.getString("fechaBusqueda"));
				} else if (datos.has("estudianteBusqueda")) {
					pst.setString(c++, "%" + datos.getString("estudianteBusqueda") + "%");
				}
				rSet = pst.executeQuery();
				rSet.next();
				rs = rSet.getInt(1);
			}
		} catch (Exception e) {
			e.printStackTrace();
			response.setStatus(false);
			response.setMessage("Error --->" + e.getMessage());
		} finally {
			try {
				if (rSet != null) {
					rSet.close();
				}
				if (pst != null) {
					pst.close();
				}
				if (con != null) {
					con.close();
				}
			} catch (Exception e) {
				e.printStackTrace();
				response.setStatus(false);
				response.setMessage("Error --->" + e.getMessage());
			}
		}
		jreturn = new JSONObject(response);
		jreturn.put("data", lista);
		jreturn.put("recordsFiltered", rs);
		jreturn.put("recordsTotal", rs);
		jreturn.put("draw", datos.getInt("draw"));
		return jreturn;
	}

	@Override
	public JSONObject listarEstudiantes(JSONObject datos) throws Exception {
		JSONObject jreturn;
		JSONArray lista = new JSONArray();
		JSONObject data = new JSONObject();
		Connection con = null;
		PreparedStatement pst = null;
		ResultSet rSet = null;
		int rs = 0;
		String sql = "";
		ResponseHelper response = new ResponseHelper();
		try {
			con = MySqlDAOFactory.obtenerConexion(GeneralVariables.nameDB);
			sql = " SELECT  "
				+ "	e.codestudiante, "
				+ "	CONCAT_WS(' ',e.nombres,e.paterno,e.materno) "
				+ " FROM estudiante AS e  "
				+ " WHERE e.codsede = ? "
				+ " AND CONCAT_WS(' ',e.nombres,e.paterno,e.materno) LIKE ? "
				+ " OR e.dni LIKE ? "
				+ " LIMIT ?,? ";
			pst = con.prepareStatement(sql);
			pst.setInt(1, datos.getInt("codigoSede"));
			pst.setString(2, "%" + datos.getString("search") + "%");
			pst.setString(3, "%" + datos.getString("search") + "%");
			pst.setInt(5, GeneralVariables.length_select2);
			pst.setInt(4, (datos.getInt("page") - 1) * GeneralVariables.length_select2);
			rSet = pst.executeQuery();
			while (rSet.next()) {
				JSONObject obj = new JSONObject();
				obj.put("id", rSet.getInt(1));
				obj.put("nombre", rSet.getString(2));
				lista.put(obj);
			}
			sql = " SELECT  "
				+ "	COUNT(*) "
				+ " FROM estudiante AS e  "
				+ " WHERE e.codsede = ? "
				+ " AND CONCAT_WS(' ',e.nombres,e.paterno,e.materno) LIKE ? "
				+ " OR e.dni LIKE ? ";
			pst = con.prepareStatement(sql);
			pst.setInt(1, datos.getInt("codigoSede"));
			pst.setString(2, "%" + datos.getString("search") + "%");
			pst.setString(3, "%" + datos.getString("search") + "%");
			rSet = pst.executeQuery();
			rSet.next();
			rs = rSet.getInt(1);
		} catch (Exception e) {
			e.printStackTrace();
			response.setStatus(false);
			response.setMessage("Error --->" + e.getMessage());
		} finally {
			try {
				if (rSet != null) {
					rSet.close();
				}
				if (pst != null) {
					pst.close();
				}
				if (con != null) {
					con.close();
				}
			} catch (Exception e) {
				e.printStackTrace();
				response.setStatus(false);
				response.setMessage("Error --->" + e.getMessage());
			}
		}
		data.put("total_count", rs);
		data.put("incomplete_results", true);
		data.put("items", lista);
		return data;
	}

	@Override
	public JSONObject registrarPermisos(JSONObject datos) throws Exception {
		JSONObject jreturn;
		JSONArray lista = new JSONArray();
		JSONObject data = new JSONObject();
		Connection con = null;
		PreparedStatement pst = null;
		ResultSet rSet = null;
		int rs = 0;
		String sql = "";
		ResponseHelper response = new ResponseHelper();
		try {
			con = MySqlDAOFactory.obtenerConexion(GeneralVariables.nameDB);
			con.setAutoCommit(false);
			sql = " INSERT INTO permisos ( "
				+ "	codigo_estudiante, "
				+ "	descripcion_permiso, "
				+ "	fecha_inicio_permiso, "
				+ "	fecha_fin_permiso "
				+ ") "
				+ "VALUES "
				+ "	(?,?,STR_TO_DATE(?,'%d/%m/%Y'),STR_TO_DATE(?,'%d/%m/%Y')) ";
			pst = con.prepareStatement(sql);
			int c = 1;
			pst.setInt(c++, datos.getInt("codigoEstudiante"));
			pst.setString(c++, datos.getString("descripcionPermiso"));
			pst.setString(c++, datos.getString("inicioPermiso"));
			pst.setString(c++, datos.getString("finPermiso"));
			rs = pst.executeUpdate();
			if (rs != 1) {
				response.setStatus(false);
				response.setMessage("Ocurrio un error al registrar el Permiso");
			} else {
				response.setStatus(true);
				response.setMessage("Inserción correcta");
				con.commit();
			}
		} catch (Exception e) {
			e.printStackTrace();
			response.setStatus(false);
			response.setMessage("Error --->" + e.getMessage());
		} finally {
			try {
				if (rSet != null) {
					rSet.close();
				}
				if (pst != null) {
					pst.close();
				}
				if (con != null) {
					con.close();
				}
			} catch (Exception e) {
				e.printStackTrace();
				response.setStatus(false);
				response.setMessage("Error --->" + e.getMessage());
			}
		}
		jreturn = new JSONObject(response);
		return jreturn;
	}

	@Override
	public JSONObject eliminarPermiso(JSONObject datos) throws Exception {
		JSONObject jreturn;
		JSONArray lista = new JSONArray();
		JSONObject data = new JSONObject();
		Connection con = null;
		PreparedStatement pst = null;
		ResultSet rSet = null;
		int rs = 0;
		String sql = "";
		ResponseHelper response = new ResponseHelper();
		try {
			con = MySqlDAOFactory.obtenerConexion(GeneralVariables.nameDB);
			con.setAutoCommit(false);
			sql = " UPDATE permisos SET estado_registro = 0 "
				+ " WHERE codigo_permiso = ?  ";
			pst = con.prepareStatement(sql);
			pst.setInt(1, datos.getInt("codigoPermiso"));
			rs = pst.executeUpdate();
			if (rs != 1) {
				response.setStatus(false);
				response.setMessage("Ocurrio un error al eliminar el Permiso");
			} else {
				response.setStatus(true);
				response.setMessage("Permiso correctamente eliminado");
				con.commit();
			}
		} catch (Exception e) {
			e.printStackTrace();
			response.setStatus(false);
			response.setMessage("Error --->" + e.getMessage());
		} finally {
			try {
				if (rSet != null) {
					rSet.close();
				}
				if (pst != null) {
					pst.close();
				}
				if (con != null) {
					con.close();
				}
			} catch (Exception e) {
				e.printStackTrace();
				response.setStatus(false);
				response.setMessage("Error --->" + e.getMessage());
			}
		}
		jreturn = new JSONObject(response);
		return jreturn;
	}

	@Override
	public JSONObject editarPermiso(JSONObject datos) throws Exception {
		JSONObject jreturn;
		JSONArray lista = new JSONArray();
		JSONObject data = new JSONObject();
		Connection con = null;
		PreparedStatement pst = null;
		ResultSet rSet = null;
		int rs = 0;
		String sql = "";
		ResponseHelper response = new ResponseHelper();
		try {
			con = MySqlDAOFactory.obtenerConexion(GeneralVariables.nameDB);
			con.setAutoCommit(false);
			sql = " UPDATE permisos  "
				+ "SET  "
				+ "	descripcion_permiso = ?, "
				+ "	fecha_inicio_permiso = STR_TO_DATE(?,'%d/%m/%Y'), "
				+ "	fecha_fin_permiso = STR_TO_DATE(?,'%d/%m/%Y') "
				+ "WHERE codigo_permiso = ? ";
			pst = con.prepareStatement(sql);
			int c = 1;
			pst.setString(c++, datos.getString("descripcionPermiso"));
			pst.setString(c++, datos.getString("inicioPermiso"));
			pst.setString(c++, datos.getString("finPermiso"));
			pst.setInt(c++, datos.getInt("codigoPermiso"));
			rs = pst.executeUpdate();
			if (rs != 1) {
				response.setStatus(false);
				response.setMessage("Ocurrio un error al actualizar el Permiso");
			} else {
				response.setStatus(true);
				response.setMessage("Permiso correctamente modificado");
				con.commit();
			}
		} catch (Exception e) {
			e.printStackTrace();
			response.setStatus(false);
			response.setMessage("Error --->" + e.getMessage());
		} finally {
			try {
				if (rSet != null) {
					rSet.close();
				}
				if (pst != null) {
					pst.close();
				}
				if (con != null) {
					con.close();
				}
			} catch (Exception e) {
				e.printStackTrace();
				response.setStatus(false);
				response.setMessage("Error --->" + e.getMessage());
			}
		}
		jreturn = new JSONObject(response);
		return jreturn;
	}

}
